

(eval-when (load eval compile)
 (require 'bordeaux-threads)
 (require 'usocket))

(defparameter *lock* (bt:make-recursive-lock "proc-lock"))
(defparameter *max-procs* 20) ;; max number of threads we allow

(let ((nprocs 0)) ;; counter for how many threads are running
  (defun safe-make-process (func)
    "A function that launches FUNC in a new thread only if 
there are not more than *MAX-PROCS* running; otherwise it sleeps
for a bit"
    ;; sleep until some processes die - print warning whenever
    ;; we're using too many procecces
    (loop
       while
	 (bt:with-lock-held (*lock*)
	   (> nprocs *max-procs*))
       do
	 (bt:with-lock-held (*lock*)
	   (format t "Warning - nprocs=~A~%" nprocs) )
	 (force-output)
	 (sleep 0.1))
    ;;
    (bt:with-lock-held (*lock*)
      (incf nprocs))
    (bt:make-thread 
     (lambda ()
       (unwind-protect ;; protect the decrement of the counter
	    (funcall func)
	 (bt:with-lock-held (*lock*)
	   (decf nprocs)))))))
  
  



(defun do-dumb-server-loop (&key (port 9001))
  (loop
     with listener = (usocket:socket-listen "127.0.0.1" port 
					    :backlog 10)
     for sock = (usocket:socket-accept listener)
     when sock
     do
       (safe-make-process
	(lambda ()
	  ;; I think we need to close over sock here or this thread's
	  ;; loop might change what REPLY-FUNC sees
	  (let ((this-sock sock)) 
	    (reply-func this-sock)))
       )))


(defparameter *response*
"HTTP/1.1 200 OK
Server: dumb
Connection: Close
Content-Type: text/plain

This is the content.
")

(defun reply-func (sock)
  (let ((s (usocket:socket-stream sock)))
    (ignore-errors
      ;; eat client headers
      (loop for line = (read-line s nil nil) 
	 until (< (length line) 3))
      ;; and send response
      (write-string *response* s))
    ;; the socket close is protected by ignore-errors
    (ignore-errors (usocket:socket-close sock))))
       


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; ccl version that does not use bordeaux-threads or usocket

(let ((nprocs 0)) ;; counter for how many threads are running
  (defun safe-make-process/ccl (func)
    "A function that launches FUNC in a new thread only if 
there are not more than *MAX-PROCS* running; otherwise it sleeps
for a bit"
    ;; sleep until some processes die - print warning whenever
    ;; we're using too many procecces
    (loop
       while
	 (bt:with-lock-held (*lock*)
	   (> nprocs *max-procs*))
       do
	 (bt:with-lock-held (*lock*)
	   (format t "Warning - nprocs=~A~%" nprocs) (force-output))
	 (sleep 0.1))
    ;;
    (bt:with-lock-held (*lock*)
      (incf nprocs))
    (ccl:process-run-function "ccl-dumb-server-process"
     (lambda ()
       (unwind-protect ;; protect the decrement of the counter
	    (funcall func)
	 (bt:with-lock-held (*lock*)
	   (decf nprocs)))))))
  


(defun do-dumb-server-loop/ccl (&key (port 9001))
  (loop
     with listener = (ccl:make-socket   
		      :connect :passive
		      :local-port port
		      :backlog 10
		      :local-host (ccl:dotted-to-ipaddr "127.0.0.1"))
     for stream = (ccl:accept-connection listener)
     when stream
     do
       (safe-make-process/ccl  
	(lambda ()
	  ;; close over stream
	  (let ((this-stream stream)) 
	    (reply-func/ccl this-stream)))
       )))


(defun reply-func/ccl (s)
  (ignore-errors
    ;; eat client headers
    (loop for line = (read-line s nil nil) 
       until (< (length line) 3))
    ;; and send response
    (write-string *response* s))
  ;; the socket close is protected by ignore-errors
  (ignore-errors (close s)))
       